// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "display/shader/shaders/SimpleBillboardedShader.hpp"
#include "SimpleBillboardedMaterial.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

SimpleBillboardedMaterial::SimpleBillboardedMaterial(SceneItem *i) :
  Material(i)
{
}

void SimpleBillboardedMaterial::onStart() {
  this->shaderLock = this->getGame()->renderManager.getShaderManager()->lockShader<SimpleBillboardedShader>();
}

void SimpleBillboardedMaterial::onDispose() {
  this->getGame()->renderManager.getShaderManager()->releaseShader<SimpleBillboardedShader>(this->shaderLock);
}

std::vector<struct ShaderPassItem> SimpleBillboardedMaterial::getRenderPasses(IRenderableContext &context) {
  auto mesh = this->item->getComponent<MeshRenderer>();
  auto shader = this->getGame()->renderManager.getShaderManager()->getShader<SimpleBillboardedShader>(this->shaderLock);

  assertNotNull(mesh, "SimpleBillboardedMaterial::getRenderPasses: Mesh cannot be null");
  assertNotNull(mesh->mesh, "SimpleBillboardedMaterial::getRenderPasses: Mesh cannot be null");
  assertNotNull(shader, "SimpleBillboardedMaterial::getRenderPasses: Shader cannot be null");

  struct ShaderPassItem onlyPass;
  onlyPass.mesh = mesh->mesh;
  onlyPass.shader = shader;
  onlyPass.colorValues[shader->paramColor] = this->color;
  onlyPass.matrixValues[shader->paramModel] = this->transform->getWorldTransform();
  onlyPass.parameterBuffers[shader->bufferRenderPipeline] = &context.renderPipeline->shaderBuffer;

  onlyPass.renderFlags = (
    RENDER_MANAGER_RENDER_FLAG_BLEND
  );

  if(this->texture != nullptr) {
    onlyPass.boolValues[shader->paramHasTexture] = true;
    onlyPass.textureSlots[0] = this->texture;
    onlyPass.textureValues[shader->paramTexture] = 0;
  } else {
    onlyPass.boolValues[shader->paramHasTexture] = false;
  }
  
  return { onlyPass };
}